@@ -4,23 +4,28 @@ import asyncio
 import copy
 import os
 import socket
+import time
 from functools import partial
-from typing import List, Callable, Any, Type
+from typing import List, Callable, Any, Type, Dict
 
 import loguru
+import orjson
 from dependency_injector import providers
 from pydantic import BaseModel as PDModel
-from sanic import Sanic
+from sanic import Sanic, Request
 from sanic.server.socket import bind_socket
-
+from sanic.response import BaseHTTPResponse
 from web_foundation.app.app import AppContainer
-from web_foundation.app.events.store import StoredPluginsUpdate
 from web_foundation.app.infrastructure.mdw_manager import ApiMiddlewareManager
+from web_foundation.app.infrastructure.metrics.formatter import GenMetricFormatter, JsonMetricFormatter, MetricFormatter
+from web_foundation.app.infrastructure.metrics.mixin import ApiMetricsMixin
 from web_foundation.kernel import GenericIMessage, IMessage
 from web_foundation.workers.io.http.ext_error_handler import ExtendedErrorHandler
 from web_foundation.workers.io.http.routers.ext_router import ExtRouter
 from web_foundation.workers.io.rt_connection import RtConnection, RtEventCallback
 from web_foundation.workers.worker import Worker
+from web_foundation import settings
+from web_foundation.app.infrastructure.metrics.exporter import MetricExporter, Metric
 
 
 class ServerConfig(PDModel):
@@ -28,47 +33,55 @@ class ServerConfig(PDModel):
     port: int
 
 
-class HttpServer(Worker):
+class HttpServer(Worker, ApiMetricsMixin):
     config: providers.Configuration
     sock: socket.socket
     sanic_app: Sanic
     router: ExtRouter | None
     rt_connections: List[RtConnection]
     mdw_manager: ApiMiddlewareManager | None
+    metrics_exporters: Dict[GenMetricFormatter, MetricExporter]
 
     def __init__(self, config: providers.Configuration, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.config = config
 
     def _configure(self, sock: socket.socket = None, ext_router: ExtRouter = None,
-                   mdw_manager: ApiMiddlewareManager = None):  # type: ignore
+                   mdw_manager: ApiMiddlewareManager = None,
+                   metrics_exporters: Dict[GenMetricFormatter, MetricExporter] = None):  # type: ignore
         self.sock = sock if sock else self.create_socket(self.config.get("host"), self.config.get("port"))
         self.router = copy.deepcopy(ext_router)
-        self.sanic_app = Sanic(self.name, router=self.router)
-        # self.sanic_app.extend(built_in_extensions=False)
+        self.sanic_app = Sanic(self.name, router=self.router, dumps=orjson.dumps, loads=orjson.loads)
         self.sanic_app.error_handler = ExtendedErrorHandler()
         self.sanic_app.after_server_stop(self.close)
-        self.sanic_app.ctx.channel = self.channel
         self.rt_connections = []
-        self._set_sanic_confs()
         self.mdw_manager = mdw_manager
-        if mdw_manager:
-            self.mdw_manager.debug = self.debug
+        self.metrics_exporters = metrics_exporters if metrics_exporters else {}
+        self.metrics_exporters.update({JsonMetricFormatter[Dict]: MetricExporter()})
+
+    def _post_configure(self):
+        self._set_sanic_confs()
+        if self.mdw_manager and not (self.repo or self.store):
+            raise AttributeError(f"You need FileRepository and AppStore to use MiddlewareManager")
+        if self.mdw_manager and issubclass(self.mdw_manager.__class__, ApiMiddlewareManager):
+            self.mdw_manager.store = self.store
+            self.mdw_manager.repo = self.repo
         if self.router and issubclass(self.router.__class__, ExtRouter):
-            self.router.apply_routes(self.sanic_app)
+            self.router.apply_routes(self.sanic_app, self.mdw_manager)
 
     def _set_sanic_confs(self):
         self.sanic_app.config.SWAGGER_UI_CONFIGURATION = {
             "docExpansion": 'none'
         }
 
-    async def _init_wrk(self, app):
-        await self._init_resources()
-        if self.mdw_manager:
-            await self.mdw_manager.import_middleware()
-
     async def _run(self):
-        self.sanic_app.register_listener(self._init_wrk, "before_server_start")
+        async def _init_wrk(app, loop):
+            await self._init_resources()
+            if self.mdw_manager:
+                await self.mdw_manager.discover_middleware()
+                await self.mdw_manager.import_middleware()
+
+        self.sanic_app.register_listener(_init_wrk, "before_server_start")
         # noinspection PyAsyncCall
         self.sanic_app.add_task(self.channel.listen_consume())
 
@@ -106,7 +119,7 @@ class HttpServer(Worker):
 
     async def accept_rt_connection(self, conn: RtConnection,
                                    on_disconnect_callback: Callable[[HttpServer, RtConnection], Any] | None = None):
-        conn.debug = self.debug if self.debug else conn.debug
+        # conn.debug = self.debug if self.debug else conn.debug
         self.rt_connections.append(conn)
 
         async def _on_rt_close():
@@ -156,4 +169,4 @@ def create_rt_subscribes(*workers: HttpServer,
                          **kwargs):
     for worker in workers:
         worker.channel.add_event_listener(event_type, partial(worker.broadcasts_rt, resolve_callback=resolve_callback),
-                                          **kwargs)
\ No newline at end of file
+                                          **kwargs)
